/*##############################################################
messiahLW.c
Plugins for the actual displacement within Lightwave

by Lyle Milton
03/09/02
copyright 1998-2002 Dan Milling, Lyle Milton, & Fori Owurowa

##############################################################*/

#define HOST_MODULE
#define HOST_MODULE_NAME	"messiahLW.p"
#define HOST_APP_NAME		"LightWave"

#include "messiahLW.h"

HINSTANCE					messiahlib = NULL;

//connection management globals-------------------------------------------------
static int					messiah_scene_saved=0, messiah_scene_loaded=0, master_active = 0;

static int					globals_created = 0, processed=0,
							load_counter=0, loading_scene=0,
							counter=0, point_counter=0,
							current_save_node = 0;


//-----------------------------------------------------------------------------
//struct to scan LW mesh.  This is needed only because LW doesn't let you
//scan through the points yourself; you must use a point scan callback. 
typedef struct LW_NodeData_St
{
	int				num_points;
	int				num_polys;
	int				num_surfaces;
	LWPntID			*pID;
	LWSurfaceID		*sID;
	LWItemID		ID[2];

	int				current_point;

}LW_NodeData;

static int pntScan(FX_NodeLog *nd, LWPntID id );
static int pntScanGetPoints(FX_NodeLog *nd, LWPntID id );
static int polScan(FX_NodeLog *nd, LWPolID id );

static int lwcommand(char *cmdname, char *fmt, ... );

//Dan 5-23-02
char *cApplication_Name=NULL;

static LWItemID temporary_lwID_global = LWITEM_NULL;
static int		temporary_messiahobj_global = 0;


//################################################################################
//host* functions/callbacks
//################################################################################


//host* declarations--------------------------------------------------------
__EXPORTVAR(int) hostLoadmessiahHOST(void);
__EXPORTVAR(int) hostSetContentDirectory(char *content_buffer, int buff_length, int flags);
__EXPORTVAR(int) hostObjectGetPath(FX_NodeLog *nd, char *object_path, int buff_length, int flags);
__EXPORTVAR(int) hostObjectRename(FX_NodeLog *nd, int flags);
__EXPORTVAR(int) hostObjectGeometryCopy(FX_NodeLog *nd, int add_to_hierarchy);
__EXPORTVAR(int) hostObjectGetPoints(FX_NodeLog *nd, int reset_object);
__EXPORTVAR(void) hostGlobalsGetAll(void);
__EXPORTVAR(int) hostLoadmessiahHOST(void);

__EXPORTVAR(void*) hostObjectLoad(char *object_path, int buff_length, int flags); //EXPORTED
__EXPORTVAR(void *) hostObjectLoadLayer(char *object_path, int Layer_ID, int buff_length, int flags);//EXPORTED
__EXPORTVAR(void *) hostObjectConstruct(int obj, int flags); //NOT EXPORTED - LW loads objects
__EXPORTVAR(void *) hostObjectAdd(int obj, char *name, int type, int sub_type, int flags);
__EXPORTVAR(int) hostAddPlugin(void *obj_data, int obj, int messiah_plugtype, int flags);

#define	HOST_SET(v)			if (!strcmp(#v,global_name)) return &##v
#define	MESSIAH_GET(v)		v = messiahGlobal(#v)


void ProcessLWObjInstance(const char *src, char *destname, int *instance)
{
	const char *open=NULL;
	const char *close=NULL;
	int x,len;


	destname[0]=0x00;
	*instance=0;

	if(src)
	{
		len=strlen(src);
		if(src[len-1]==')')
		{
			for(x=len-2;x>0;x--)
			{
				if(src[x]>='0' && src[x]<='9')
				{


				}
				else if(src[x]=='(')
				{
					if(x>0)
					{
						if(src[x-1]==0x20)//check for space
						{
							strncpy(destname,src,x-1);
							sscanf(&src[x+1],"%d",instance);
							break;
						}
						else
						{
							strcpy(destname,src);
							*instance=0;
							break;
						}
					}
					else
					{
						strcpy(destname,src);
						*instance=0;
						break;
					}
				}
				else
				{
					strcpy(destname,src);
					*instance=0;
					break;
				}
			}
		}
		else
		{
			strcpy(destname,src);
			*instance=0;
		}
	}


}





void ProcessLWName(const char *src, char *oname, char *lname, int *instance)
{
	const char *layer=NULL;
	int len;

	if(src)
	{
		layer=strrchr(src,':');
		if(layer==NULL)
		{
			ProcessLWObjInstance(src,oname,instance);
			lname=NULL;
		}
		else
		{
			len=layer-src;
			strncpy(oname,src,len);
			oname[len]=0x00;

			layer++; //skip the ':'
			ProcessLWObjInstance(layer,lname,instance);
		}
	}
	else
	{
		oname=NULL;
		lname=NULL;
		*instance=0;
	}

}








//---------------------------------------------------------------------------
//	Name:	hostGlobal
//	Purpose:Required global pointer send function.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void*) hostGlobal(char *global_name)
{
	if (!global_name) return NULL;

	HOST_SET(hostSetContentDirectory);
	HOST_SET(hostObjectGeometryCopy);
	HOST_SET(hostObjectGetPoints);
	HOST_SET(hostObjectRename);

	HOST_SET(hostObjectLoad);
	HOST_SET(hostObjectLoadLayer);
	HOST_SET(hostObjectAdd);
	HOST_SET(hostAddPlugin);

	HOST_SET(hostObjectGetPath);

	return NULL;
}


//---------------------------------------------------------------------------
//	Name:	hostGlobalsGetAll
//	Purpose:Required global pointer retrieval function.  Called from host.
//---------------------------------------------------------------------------
__EXPORTVAR(void) hostGlobalsGetAll(void)
{
	MESSIAH_GET(messiah_current_frame);

	MESSIAH_GET(messiah_Activated);
	MESSIAH_GET(messiahStart);
	MESSIAH_GET(messiahInterface);
	MESSIAH_GET(messiahShutdown);
	MESSIAH_GET(messiahCheckTime);
	MESSIAH_GET(messiahUpdate);

	MESSIAH_GET(messiahSceneFileCreate);
	MESSIAH_GET(messiahSceneFileWriteLine);
	MESSIAH_GET(messiahSceneFileClose);
	MESSIAH_GET(messiahSceneFileCreateBuff);
	MESSIAH_GET(messiahSceneFileLoadBuff);
	MESSIAH_GET(messiahSceneLoad);
	MESSIAH_GET(messiahSceneClear);
	MESSIAH_GET(messiahSceneFileFilterBuff);
	MESSIAH_GET(messiahSceneFileClearBuff);

	MESSIAH_GET(num_nodes);
	MESSIAH_GET(node_log);

	MESSIAH_GET(nodeLogElement);
	MESSIAH_GET(nodeGetElement);
	MESSIAH_GET(nodeGetFirst);
	MESSIAH_GET(nodeGetLast);
	MESSIAH_GET(nodeSearchElement);
	MESSIAH_GET(nodeClearElement);
	MESSIAH_GET(nodeClearAllElements);


	MESSIAH_GET(nodeObjectCreate);
	MESSIAH_GET(nodeObjectRename);

	MESSIAH_GET(nodeObjectSetFilePath); //Dan 5-3-03
	MESSIAH_GET(nodeObjectGetFileIndexFromLayerIndex); //Dan 5-8-03
	MESSIAH_GET(nodeObjectSetLayerData); //Dan 5-8-03


	MESSIAH_GET(nodeObjectNumPoints);
	MESSIAH_GET(nodeObjectCreatePoint);
	MESSIAH_GET(nodeObjectGetPoint);
	MESSIAH_GET(nodeObjectSetPoint);

	MESSIAH_GET(nodeObjectNumSurfaces);
	MESSIAH_GET(nodeObjectCreateSurface);
	MESSIAH_GET(nodeObjectSurfaceSet);
	MESSIAH_GET(nodeObjectSurfaceGet);

	MESSIAH_GET(nodeObjectNumPolygons);
	MESSIAH_GET(nodeObjectCreatePoly);
	MESSIAH_GET(nodeObjectGetPolygon);

	MESSIAH_GET(nodeObjectEnd);
	MESSIAH_GET(nodeObjectRebuild);
	MESSIAH_GET(nodeObjectAppend);


	MESSIAH_GET(nodeObjectGetMotion);
	MESSIAH_GET(nodeObjectSetMotion);
	MESSIAH_GET(nodeObjectGetWorldPos);
	MESSIAH_GET(nodeObjectGetWorldRot);
	MESSIAH_GET(nodeObjectGetWorldScale);

	MESSIAH_GET(nodeCameraGetZoom);
	MESSIAH_GET(nodeCameraSetZoom);

	MESSIAH_GET(nodeLightGetType);
	MESSIAH_GET(nodeLightSetType);
	MESSIAH_GET(nodeLightGetColor);
	MESSIAH_GET(nodeLightSetColor);
	MESSIAH_GET(nodeLightGetIntensity);
	MESSIAH_GET(nodeLightSetIntensity);

}


//---------------------------------------------------------------------------
//	Name:	hostLoadmessiahHOST
//	Purpose:Required messiahHOST module loader.  Called from host.
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostLoadmessiahHOST(void)
{
	HMODULE				hostmodule;
	char				libpath[512], curdir[512];
	int					i;

	if (messiahlib) return TRUE; //library is already loaded

	hostmodule = GetModuleHandle(HOST_MODULE_NAME); 
	GetModuleFileName(hostmodule, curdir, 512); 

	//Path to scan for messiahHOST--------------------------------------
	i = strlen(curdir);
	curdir[i-strlen(HOST_MODULE_NAME)] = '\0';

	sprintf(libpath,"%s%s",curdir,MESSIAH_MODULE_NAME);


	//load messiahHOST--------------------------------------------------
	if (messiahlib = LoadLibrary(libpath))
	{
		if (messiahGlobal = (void*) GetProcAddress(messiahlib, "messiahGlobal"))
		{
			hostGlobalsGetAll();
		}
		else
		{
			FreeLibrary(messiahlib);
			messiahlib = NULL;
			return FALSE;
		}

		return TRUE;
	}

	return FALSE;
}


//---------------------------------------------------------------------------
//	Name:	hostSetContentDirectory
//	Purpose:Required Content Directory retrieval function.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostSetContentDirectory(char *content_buffer, int buff_length, int flags)
{
	const char		*content;

	if (!content_buffer || !dr) return FALSE;

	content = (*dr)("Content");
	strncpy(content_buffer, content, buff_length);

	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostAddPlugin
//	Purpose:Add the messiah plugin to the host object.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostAddPlugin(void *obj_data, int obj, int messiah_plugtype, int flags)
{
	LWItemID		ID;
	int				ok = TRUE;

	if (!obj_data) return FALSE;


	ii = (*gb) (LWINTERFACEINFO_GLOBAL, GFUSE_TRANSIENT);


	ID = *((LWItemID*)obj_data);

	if(lwmajor<9)
	{
		if (!lwcommand("SelectItem", "%d", ID))
		{
			(*ms->error)("hostAddPlugin: SelectItem failed!", "Can't select item to apply plugin.");
			return FALSE;
		}
	}
	
	temporary_messiahobj_global = obj;


	ii = (*gb) (LWINTERFACEINFO_GLOBAL, GFUSE_TRANSIENT);



	if (messiah_plugtype==FX_NODE_DISPLACEMENT)
	{
		ok = lwcommand("ApplyServer", "%s %s", LWDISPLACEMENT_HCLASS, "messiah:displace");
	}
	else
	{
		ok = lwcommand("ApplyServer", "%s %s", LWITEMMOTION_HCLASS, "messiah:xform");
	}

	temporary_messiahobj_global = 0;


	return ok;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectLoad
//	Purpose:Load a file object into the host.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void *) hostObjectLoad(char *object_path, int buff_length, int flags)
{
	LWItemID		ID, nextID;
	int res=0;

	if (!object_path) return NULL;

	if(lwmajor<9)
		res=(*evaluate)("EditObjects");

	temporary_lwID_global = LWITEM_NULL;

	if (!lwcommand("LoadObject", "%s", object_path)) return NULL;
	if (!lwcommand("SubdivisionOrder", "%d", 3))
	{
	}



	ID = (*it->first)(LWI_OBJECT, LWITEM_NULL);

	while((nextID=(*it->next)(ID)) != LWITEM_NULL) ID = nextID;

	temporary_lwID_global = ID;

	return &temporary_lwID_global;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectLoadLayer
//	Purpose:Load a file object into the host.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void *) hostObjectLoadLayer(char *object_path, int Layer_ID, int buff_length, int flags)
{
	LWItemID		ID, nextID;
	int res=0;


	if (!object_path) return NULL;

	if(lwmajor<9)
		res=(*evaluate)("EditObjects");

	temporary_lwID_global = LWITEM_NULL;

	ii = (*gb) (LWINTERFACEINFO_GLOBAL, GFUSE_TRANSIENT);


	if (!lwcommand("LoadObjectLayer", "%d %s", Layer_ID, object_path)) return NULL;
	if (!lwcommand("SubdivisionOrder", "%d", 3))
	{
	}


	ii = (*gb) (LWINTERFACEINFO_GLOBAL, GFUSE_TRANSIENT);


	ID = (*it->first)(LWI_OBJECT, LWITEM_NULL);

	while((nextID=(*it->next)(ID)) != LWITEM_NULL) ID = nextID;

//	lwcommand("", "%f, %f, %f, 
	temporary_lwID_global = ID;

	return &temporary_lwID_global;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectSave
//	Purpose:Save a file from a host object.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectSave(FX_NodeLog *nd, char *object_path, int buff_length, int flags)
{
	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectConstruct
//	Purpose:Build an object inside the host from the messiah obj.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void *) hostObjectConstruct(int obj, int flags)
{
	return NULL;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectAdd
//	Purpose:Add a null, light, or camera inside the host from the messiah obj.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(void *) hostObjectAdd(int obj, char *name, int type, int sub_type, int flags)
{
	LWItemID		ID, nextID;

	if (!name) return NULL;

	temporary_lwID_global = LWITEM_NULL;


	switch(type)
	{
		case FX_NODE_NULL:
			if (!lwcommand("AddNull", "%s", name)) return NULL;
			ID = (*it->first)(LWI_OBJECT, LWITEM_NULL);
			while((nextID=(*it->next)(ID)) != LWITEM_NULL) ID = nextID;
			break;

		case FX_NODE_CAMERA:
			if (!lwcommand("AddCamera", "%s", name)) return NULL;
			ID = (*it->first)(LWI_CAMERA, LWITEM_NULL);
			while((nextID=(*it->next)(ID)) != LWITEM_NULL) ID = nextID;

			if (obj)
			{
				double zoom = 0;

				nodeCameraGetZoom(obj, &zoom);

				lwcommand("ZoomFactor", "%f", zoom);
			}

			break;

		case FX_NODE_LIGHT:
			switch(sub_type)
			{
				case FX_NLIGHT_DIRECTIONAL:
					if (!lwcommand("AddDistantLight", "%s", name)) return NULL;
					break;

				case FX_NLIGHT_POINT:
					if (!lwcommand("AddPointLight", "%s", name)) return NULL;
					break;

				case FX_NLIGHT_SPOT:
					if (!lwcommand("AddSpotlight", "%s", name)) return NULL;
					break;

//				case FX_NLIGHT_SPHERE:
//					if (!lwcommand("AddSphereLight", "%s", name)) return NULL;
//					break;

				case FX_NLIGHT_PANEL:
					if (!lwcommand("AddAreaLight", "%s", name)) return NULL;
					break;

				default:
					(*ms->error)("hostObjectAdd: Light Add failed!", "messiah light type not available in LightWave.");
					return NULL;
					break;
			}

			ID = (*it->first)(LWI_LIGHT, LWITEM_NULL);
			while((nextID=(*it->next)(ID)) != LWITEM_NULL) ID = nextID;

			if (obj)
			{
				float color[3]={0,0,0};
				double intensity=0;

				nodeLightGetColor(obj, color);
				nodeLightGetIntensity(obj, &intensity);

				lwcommand("LightColor", "%f %f %f", (double)color[0], (double)color[1], (double)color[2]);
				lwcommand("LightIntensity", "%f", intensity);
			}

			break;

		default:
			(*ms->error)("hostObjectAdd: Object Add failed!", "messiah object type not available in LightWave.");
			return NULL;
			break;
	}

	temporary_lwID_global = ID;

	return &temporary_lwID_global;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectConstruct
//	Purpose:Build an object inside the host from the messiah obj.  Called from messiah.
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectGetPath(FX_NodeLog *nd, char *object_path, int buff_length, int flags)
{
	LWItemID		*element = NULL;
	const char		*path;

	if (!object_path) return FALSE;

	memset(object_path, 0, buff_length);

	if (!nd) return FALSE;
	if (!(element=(LWItemID*)nd->element) || !nd->obj) return FALSE;

	path = (*ob->filename)(*element);

	if (path)
	{
		strncpy(object_path, path, buff_length);
	}
	else
	{
		return FALSE;
	}

	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectRename
//	Purpose:Required host object rename function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectRename(FX_NodeLog *nd, int flags)
{
	LWItemID		*element = NULL;
	char			*name=NULL;

	if (!(element=(LWItemID*)nd->element) || !nd->obj) return FALSE;
	name = (char*)(*it->name)(*element);

	nodeObjectRename(nd->obj, name, flags);

	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectGeometryCopy
//	Purpose:Required host object geometry creation function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectGeometryCopy(FX_NodeLog *nd, int add_to_hierarchy)
{
	LWItemID		*element = NULL;
	LWMeshInfo		*mesh=NULL;
	LWSurfaceID		surfid;
	LW_NodeData		*ndata=nd->data;
	LWItemType		item_type;

	int				num_points;
	double			*dval;
	float			color[3];


	const char		*path=NULL;

	int x, y, numobj;
	const char *layerpath=NULL;
	int maxlayers;
	const char *temp_name=NULL;

	const char *lname=NULL;

	char obj_name[512]="";
	char layer_name[512]="";
	int obj_instance=0;

	int layer_index=0;
	int file_index=0;
	int result;

//	LWDVector		pivot_point = {0.0f,0.0f,0.0f}; //Dan 3-13-03


	if (!ndata || !(element=(LWItemID*)nd->element) || !nd->obj) return FALSE;

	item_type = (*it->type)(*element);

//	(*it->param)(*element, LWIP_PIVOT, 0, pivot_point); //Dan 3-13-03 - get the pivot point

	path = ob->filename(*element);




	if ((item_type==LWI_OBJECT) && (mesh = nd->ref = ob->meshInfo( *element, 0)))
	{
		//DAN'S CODE: for getting the proper layer==========================================
		lname = (*it->name)(*element);
		ProcessLWName(lname,obj_name,layer_name,&obj_instance);

		if(layer_name[0])
		{
			if(strncmp(layer_name,"Layer",5)==0)
			{
				sscanf(&layer_name[5],"%d",&layer_index);
				result=nodeObjectGetFileIndexFromLayerIndex(nd->obj, (char *)path, &file_index, layer_index);

				nodeObjectSetLayerData(nd->obj, file_index, layer_index, layer_name);

			}
			else
			{
				numobj=(*obf->numObjects)();
				for(x=0;x<numobj;x++)
				{
					layerpath=(*obf->filename)(x);
					if(strcmp(path,layerpath)==0)
					{
						maxlayers=(*obf->maxLayers)(x);
						if(maxlayers>1)
						{

							for(y=0;y<maxlayers;y++)
							{
								temp_name=(*obf->layerName)(x,y);
								if(temp_name)
								{
									if(strcmp(temp_name,layer_name)==0)
									{
										layer_index=y+1; //add 1 to the layer index
										result=nodeObjectGetFileIndexFromLayerIndex(nd->obj, (char *)path, &file_index, layer_index);

										nodeObjectSetLayerData(nd->obj, file_index, layer_index, layer_name);

										break;
									}
								}
							}
						}
						break;
					}
				}
			}
		}
		//DAN'S CODE END=====================================================================

		
		if (!(num_points = mesh->numPoints(mesh))) return TRUE;

		if (ndata->pID)
		{
			free(ndata->pID);
			ndata->pID = NULL;
		}

		if (!(ndata->pID = calloc(num_points, sizeof(LWPntID)))) return FALSE;

		//Record the points,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		ndata->num_points=0;
		if ( mesh->scanPoints( mesh, pntScan, nd )) return FALSE;



		//modify position here. - Dan 3-13-03
//		RePositionObject(nd->obj, pivot_point);



		//Record the surface,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		//NOTE: Surfaces must always be recorded before polygons
		if (ndata->sID = sff->byObject( ob->filename(*element) ))
		{
			ndata->num_surfaces = 0;

			while(surfid=ndata->sID[ndata->num_surfaces])
			{
				nodeObjectCreateSurface(nd->obj, (char*)sff->name(surfid));

				dval = sff->getFlt( surfid, SURF_COLR );
				color[ 0 ] = ( float ) dval[ 0 ];
				color[ 1 ] = ( float ) dval[ 1 ];
				color[ 2 ] = ( float ) dval[ 2 ];
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Ambient", color);

				dval = sff->getFlt( surfid, SURF_DIFF );
				color[ 0 ] = ( float ) *dval;
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Diffuse", color);

				dval = sff->getFlt( surfid, SURF_LUMI );
				color[ 0 ] = ( float ) *dval;
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Luminous", color);

				dval = sff->getFlt( surfid, SURF_SPEC );
				color[ 0 ] = ( float ) *dval;
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Specular", color);

				dval = sff->getFlt( surfid, SURF_GLOS );
				color[ 0 ] = ( float ) (*dval * 100.0);
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Shininess", color);

				dval = sff->getFlt( surfid, SURF_TRAN );
				color[ 0 ] = ( float ) *dval;
				nodeObjectSurfaceSet(nd->obj, ndata->num_surfaces, "Transparency", color);

				//LYLETODO: Get the additional LW surface info

				ndata->num_surfaces++;
			}
		}
		else
		{
			//there must be at least one surface
			ndata->num_surfaces = 1;
			nodeObjectCreateSurface(nd->obj, (char*)it->name(*element));
		}

		//Record the polygons,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		if ( mesh->scanPolys( mesh, polScan, nd )) return FALSE;

		nodeObjectEnd(nd->obj, add_to_hierarchy);

		nodeObjectSetFilePath(nd->obj,path,0); //Dan 5-3-03

	}
	else
	{
		if (item_type==LWI_CAMERA)
		{
		}
		else if (item_type==LWI_LIGHT)
		{
		}

		nodeObjectEnd(nd->obj, add_to_hierarchy);
		//nodeObjectAppend(nd->obj, add_to_hierarchy);

		return TRUE;
	}


	return TRUE;
}


//---------------------------------------------------------------------------
//	Name:	hostObjectGetPoints
//	Purpose:Required host object point modification function
//---------------------------------------------------------------------------
__EXPORTVAR(int) hostObjectGetPoints(FX_NodeLog *nd, int reset_object)
{
	LWItemID		*element = NULL;
	LWMeshInfo		*mesh=NULL;
	LW_NodeData		*ndata=nd->data;
	LWItemType		item_type;

	int				num_points;

	if (!ndata || !(element=(LWItemID*)nd->element) || !nd->obj) return FALSE;

	item_type = (*it->type)(*element);

	nd->counter = 0;

	if ((item_type==LWI_OBJECT) && (mesh = nd->ref = ob->meshInfo( *element, 0)))
	{
		if (!(num_points = mesh->numPoints(mesh))) return FALSE;
		if ( mesh->scanPoints( mesh, pntScanGetPoints, nd )) return FALSE;
	}

	return TRUE;
}






//################################################################################
//LW mesh scan functions
//################################################################################


//---------------------------------------------------------------------------
//	Name:	pntScan
//	Purpose: Create the points and collect the point IDs
//---------------------------------------------------------------------------
static int pntScan(FX_NodeLog *nd, LWPntID id )
{
	LWMeshInfo		*mesh=nd->ref;
	LW_NodeData		*ndata=nd->data;
	LWFVector		point;

	mesh->pntBasePos( mesh, id, point);
	nodeObjectCreatePoint(nd->obj, point[0], point[1], point[2]);
	ndata->pID[ndata->num_points++] = id;
	return 0;
}


//---------------------------------------------------------------------------
//	Name:	pntScanGetPoints
//	Purpose:Get the points from the LW object
//---------------------------------------------------------------------------
static int pntScanGetPoints(FX_NodeLog *nd, LWPntID id )
{
	LWMeshInfo		*mesh=nd->ref;
	LW_NodeData		*ndata=nd->data;
	LWFVector		point;

	mesh->pntOtherPos( mesh, id, point);
	nodeObjectSetPoint(nd->obj, nd->counter++, point[0], point[1], point[2]);
	return 0;
}


//---------------------------------------------------------------------------
//	Name:	polScan
//	Purpose:Create the polygons
//---------------------------------------------------------------------------
static int polScan(FX_NodeLog *nd, LWPolID id )
{
	LWMeshInfo		*mesh=nd->ref;
	LW_NodeData		*ndata=nd->data;
	LWPntID			ptid;
	const char		*sname, *sidname;
	int				pnt[4], *p;
	int				nverts, i, j, allocated = 0, s=0;

	if (!(nverts = mesh->polSize( mesh, id ))) return 0;

	//Use the proper point array,,,,,,,,,,,,,,,,,,,,,,,,,,,
	if (nverts<5)
	{
		p = pnt;
	}
	else
	{
		p = calloc(nverts, sizeof(int));
		allocated = 1;
	}

	//Get the point IDs for this poly,,,,,,,,,,,,,,,,,,,,,,,
	for (i=0; i<nverts; i++)
	{
		ptid = mesh->polVertex( mesh, id, i );

		for (j=0;j<ndata->num_points;j++)
		{
			if (ndata->pID[j]==ptid)
			{
				p[i] = j;
				break;
			}
		}
	}

	//Get the surface ID for this poly,,,,,,,,,,,,,,,,,,,,,
	sname = mesh->polTag( mesh, id, LWPTAG_SURF );
	if (sname)
	{
		for (j=0;j<ndata->num_surfaces;j++)
		{
			if (sidname = sff->name(ndata->sID[j]))
			{
				if (!strcmp(sidname, sname))
				{
					s = j;
					break;
				}
			}
		}
	}

	//create the poly,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	//NOTE: surfaces IDs start at 1, not 0.  This is the reason we always add 1 to the ID.
	//This note is only for polys.  Start at 0 when referencing surfaces directly, as 
	//demonstrated in the nodeObjectSurface* functions.
	nodeObjectCreatePoly(nd->obj, nverts, p, (unsigned short)(s+1), 0);

	if (allocated) free(p);

	return 0;
}






//################################################################################
//Utility functions
//################################################################################



//---------------------------------------------------------------------------
//	Name:	GetLWGlobals
//	Purpose:Get LightWave globals & start messiah
//---------------------------------------------------------------------------
static int GetLWGlobals(GlobalFunc *global)
{
	static int		set_globals = 1;

	if (!set_globals) return AFUNC_OK;
	set_globals = 0;

	gb = global;


	//The messages global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (!(ms = (*gb) (LWMESSAGEFUNCS_GLOBAL, GFUSE_TRANSIENT)))
		return (AFUNC_BADGLOBAL);

	//The directory global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if(!(dr = (*gb)(LWDIRINFOFUNC_GLOBAL,GFUSE_TRANSIENT)))
	{
		(*ms->error)("Unable to get Lightwave's Directory Information!", NULL );
		return AFUNC_BADGLOBAL;
	}

	//The object global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(obf = (*gb) (LWOBJECTFUNCS_GLOBAL, GFUSE_TRANSIENT)))
	{
		if(!(obf = (*gb) ("Scene Objects 3", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Can't access Object Funcs",NULL);
			return (AFUNC_BADGLOBAL);
		}
	}

	//The surfaces global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(sff = (*gb) (LWSURFACEFUNCS_GLOBAL, GFUSE_TRANSIENT)))
	{
		(*ms->error)("Can't access Surface Funcs",NULL);
		return (AFUNC_BADGLOBAL);
	}

	//The object info global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(ob = (*gb) (LWOBJECTINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		if(!(ob = (*gb) ("LW Object Info 3", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Can't access Object Info",NULL);
			return (AFUNC_BADGLOBAL);
		}
	}

	//The item global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(it = (*gb) (LWITEMINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		(*ms->error)("Can't access Item Info",NULL);
		return (AFUNC_BADGLOBAL);
	}
	
	//The camera global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(ci = (*gb) (LWCAMERAINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		if(!(ci = (*gb) ("LW Camera Info 2", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Can't access Camera Info",NULL);
			return (AFUNC_BADGLOBAL);
		}
	}
	
	//The scene global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(si = (*gb) (LWSCENEINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		if(!(si = (*gb) ("LW Scene Info 2", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Unable to access scene information", NULL );
			return AFUNC_BADGLOBAL;
		}
	}

	//The bone info global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(bi = (*gb) (LWBONEINFO_GLOBAL, GFUSE_TRANSIENT)))
	{ 
		if(!(bi = (*gb) ("LW Bone Info 2", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Unable to access bone information", NULL );
			return AFUNC_BADGLOBAL;
		}
	}

	//The light global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(li = (*gb) (LWLIGHTINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		if(!(li = (*gb) ("LW Light Info 2", GFUSE_TRANSIENT)))
		{
			(*ms->error)("Unable to access light information", NULL );
			return AFUNC_BADGLOBAL;
		}
	}


	prodinfo = ( unsigned long ) global( LWPRODUCTINFO_GLOBAL, GFUSE_TRANSIENT );
	lwproduct = prodinfo & LWINF_PRODUCT;
	lwmajor = LWINF_GETMAJOR( prodinfo );
	lwminor = LWINF_GETMINOR( prodinfo );
	lwbuild = LWINF_GETBUILD( prodinfo );


	//The host display global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if(!(ii = (*gb) (LWINTERFACEINFO_GLOBAL, GFUSE_TRANSIENT)))
	{
		(*ms->error)("Unable to access interface information", NULL );
		return AFUNC_BADGLOBAL;
	}

	//The command global~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (!(evaluate = (*gb) ( LWCOMMANDFUNC_GLOBAL, GFUSE_TRANSIENT )))
	{
		//(*ms->error)("Unable to access command information", NULL );
		//return AFUNC_BADGLOBAL;
	}

/*
	//Start up the messiah manager=====================================================
	if (!messiahStart(HOST_APP_NAME, HOST_MODULE_NAME, hostGlobal, MESSIAH_HOSTAPI_VERSION))
	{
		(*ms->error)("Can't start messiah!", NULL);
		return AFUNC_BADGLOBAL;
	}
*/
	return AFUNC_OK;
}


//---------------------------------------------------------------------------
//	Name:	lwcommand
//	Purpose:Issue LW command
//---------------------------------------------------------------------------
static int lwcommand(char *cmdname, char *fmt, ... )
{
  static char cmd[ 512 ], arg[ 512 ];

  if ( fmt )
  {
     va_list ap;
     va_start( ap, fmt );
     vsprintf( arg, fmt, ap );
     va_end( ap );

     sprintf( cmd, "%s %s", cmdname, arg );
     return (*evaluate)( cmd );
  }
  else
     return (*evaluate)( cmdname );
}



//---------------------------------------------------------------------------
//	Name:	MasterShutdown
//	Purpose:remove the .messiah:master
//---------------------------------------------------------------------------
void MasterShutdown(void)
{
	const char	*server_name = (const char *)-1;
	int			i, v;

	if (!master_active) return;

	if (!evaluate) return;

	for ( i=1; server_name != NULL; i++ )
	{
		server_name = (*it->server)( NULL, LWMASTER_HCLASS, i );

		if( server_name == NULL ) break;

		if(!strcmp( server_name, "messiah:master" ))
		{
			v = lwcommand("RemoveServer", "%s %d", LWMASTER_HCLASS, i);
			//return;
		}
	}

	master_active = 0;
}


//---------------------------------------------------------------------------
//	Name:	MasterDestroyDuplicates
//	Purpose:remove duplicate .messiah:masters
//---------------------------------------------------------------------------
void MasterDestroyDuplicates(void)
{
	const char	*server_name = (const char *)-1;
	int			i, v, exists = 0;

	if (!master_active) return;

	if (!evaluate) return;

	for ( i=1; server_name != NULL; i++ )
	{
		server_name = (*it->server)( NULL, LWMASTER_HCLASS, i );

		if( server_name == NULL ) break;

		if(!strcmp( server_name, "messiah:master" ))
		{
			if (exists)
			{
				v = lwcommand("RemoveServer", "%s %d", LWMASTER_HCLASS, i);
			}
			else exists = 1;
		}
	}
}


//---------------------------------------------------------------------------
//	Name:	MasterStart
//	Purpose:apply the .messiah:master - necessary to monitor commands
//---------------------------------------------------------------------------
void MasterStart(void)
{
	if (!evaluate) return;

	if (!master_active)
	{
//		lwcommand("ApplyServer", "%s %s", LWMASTER_HCLASS, "messiah:master");
		master_active = 1;
	}
	else //get rid of duplicates
	{
		MasterDestroyDuplicates();
	}
}


//---------------------------------------------------------------------------
//	Name:	NodeCreate
//	Purpose:Create a messiah Node instance
//---------------------------------------------------------------------------
XCALL_(static LWInstance)
NodeCreate (void *priv, LWItemID curItem, LWError *err, int plug_type)
{
	FX_NodeLog		*node;
	LW_NodeData		*ndata=NULL;
	char			*name;
	LWItemID		*element = NULL;
	char			*filename;
	char			newname[1024]="";
	LWItemType		item_type;
	LWMeshInfo		*mesh=NULL;
	int				notify_bad_plug = 0;

	//Start the master module,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
//	MasterStart(); //moved this to the MasterCreate function


	//Start up the messiah manager=====================================================
	if (!messiahStart(HOST_APP_NAME, HOST_MODULE_NAME, hostGlobal, MESSIAH_HOSTAPI_VERSION))
	{
		(*ms->error)("Can't start messiah!", NULL);
		return NULL;
	}


	//Allocate the holder for the LW ID,,,,,,,,,,,,,,,,,,,,,,,,,,
	if (!(element = calloc(1, sizeof(LWItemType)))) return NULL;
		*element = curItem;

	//Allocate the holder for the LW specific data,,,,,,,,,,,,,,,
	if (!(ndata = calloc(1, sizeof(LW_NodeData))))
	{
		free(element);
		return NULL;
	} 

	ndata->ID[0] = curItem;

	//Create teh node log,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	if (!(node = nodeLogElement(NULL, element, ndata, 0)))
	{
		free(ndata);
		free(element);
		return NULL;
	} 

	node->type = plug_type; 
	name = (char*)(*it->name)(curItem);
	filename = (char*)(*ob->filename) (curItem); 
	item_type = (*it->type)(curItem);

	//Using the master to determine load. This isn't possible with older versions of LW
	//so we may code a messiahLW specifically for those versions. 

	if (!temporary_messiahobj_global)
	{
		if (!loading_scene)
		{
			if ((item_type==LWI_OBJECT) && (mesh = node->ref = ob->meshInfo( curItem, 0)))
			{
				if(mesh->numPoints(mesh))
				{
					node->obj = nodeObjectCreate(name, FX_NODE_GEOMETRIC, 0);
					hostObjectGeometryCopy(node, 1);
				}
				else
				{
					if (plug_type)
					{
						node->obj = nodeObjectCreate(name, FX_NODE_NULL, 0);
						nodeObjectEnd(node->obj, 1);
					}
					else
						notify_bad_plug = 1;
				}
			} 
			else if (plug_type)
			{
				if (item_type==LWI_CAMERA)
				{
					node->obj = nodeObjectCreate(name, FX_NODE_CAMERA, 0);
					nodeObjectEnd(node->obj, 1);
				}
				else if (item_type==LWI_LIGHT)
				{
					int		l_type = (*li->type)(curItem);

					node->obj = nodeObjectCreate(name, FX_NODE_LIGHT, 0);
					nodeLightSetType(node->obj, l_type);
					nodeObjectEnd(node->obj, 1);
				}
				else
				{
					node->obj = nodeObjectCreate(name, FX_NODE_NULL, 0);
					nodeObjectEnd(node->obj, 1);
				}
			}
			else
				notify_bad_plug = 1;
		}
	}
	else
	{
		if ((item_type==LWI_OBJECT) && (mesh = node->ref = ob->meshInfo( curItem, 0)))
		{
			ndata->num_points = mesh->numPoints(mesh);
			ndata->num_polys = mesh->numPolygons(mesh);
		} 

		node->obj = temporary_messiahobj_global;
		temporary_messiahobj_global = 0;
	}

	counter++; 

	return node;
} 








//################################################################################
//LW Plugin Callbacks
//################################################################################


//---------------------------------------------------------------------------
//	Name:	NodeCreateMotion
//	Purpose:Create an messiah:xform Node instance
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static LWInstance)
NodeCreateMotion ( void *priv, LWItemID item, LWError *err )
{
	return NodeCreate (priv, item, err, FX_NODE_MOTION);
}

//---------------------------------------------------------------------------
//	Name:	NodeCreateDisplace
//	Purpose:Create an messiah:displace Node instance
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static LWInstance)
NodeCreateDisplace ( void *priv, LWItemID item, LWError *err )
{
	return NodeCreate (priv, item, err, FX_NODE_DISPLACEMENT);
}

//---------------------------------------------------------------------------
//	Name:	NodeDestroy
//	Purpose:Destroy an messiah Node instance
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static void)
NodeDestroy ( FX_NodeLog *node)
{
	LW_NodeData		*ndata=node->data;

	if (ndata)
	{
		if (ndata->pID) free(ndata->pID);
		free(ndata);
		node->data = NULL;
	}

	counter--;
	if (counter<0) counter = 0;


	if (counter)
	{
		nodeClearElement(node, 0);
	}
	else
	{
		nodeClearAllElements(2);
//		MasterShutdown();
	}

	return;
}

//---------------------------------------------------------------------------
//	Name:	NodeCopy
//	Purpose:Copy an messiah Node instance
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static LWError)
NodeCopy (
	FX_NodeLog			*from,
	FX_NodeLog			*to)
{
	//*to = *from;
	return (NULL);
}


//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#define BUFSIZE		4096
#define FXWRITE	(*sState->write) (sState->writeData, buf, strlen (buf) + 1)
#define FXREAD	(*lState->read) (lState->readData, buf, BUFSIZE)

//---------------------------------------------------------------------------
//	Name:	NodeLoad
//	Purpose:Load file
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static LWError)
NodeLoad ( FX_NodeLog *node, const LWLoadState	*lState)
{
	char		buf[BUFSIZE], tbuf[128], *m_buff=NULL;
	int			len=0, m_len=0, merge=0;

	load_counter++;

	FXREAD;
	sscanf(buf, "%s %d", tbuf, &node->load_ID); //This is necessary in case nodes are saved out of order

	FXREAD;
	if (strcmp(buf,"MESSIAH_FILE_BEGIN_BLOCK")) return NULL;

	if (!messiahSceneFileCreate(0)) return NULL;

	FXREAD;
	while(strcmp(buf,"MESSIAH_FILE_END_BLOCK"))
	{
		messiahSceneFileWriteLine(buf, 0);

		FXREAD;
	}

	if (messiahSceneFileClose(0));
	{
		if (load_counter!=counter)
			messiahSceneLoad(MESSIAH_SCENE_MERGE);
		else
			messiahSceneLoad(MESSIAH_SCENE_LOAD);

		messiahCheckTime(-292929.0, 1); //this will force an update. The time value is arbitrary
	}

	return (NULL);
}

//---------------------------------------------------------------------------
//	Name:	NodeSave
//	Purpose:Save file
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static LWError)
NodeSave ( FX_NodeLog	 *node, const LWSaveState	*sState)
{
	char		*m_buff, *buff1, *buff2;
	char		buf[BUFSIZE]; //4k buffer
	int			len;

	//### Because the LW instances may not be in the same order as the node list,
	//### we need to store the IDs and check for the last node using a counter.
	//### This means that counter needs to be initialized before saving.  It's
	//### done so in LW every chance we get because it doesn't have any PRE
	//### messaging.

	//Write the ID for this node,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	sprintf (buf, "MESSIAH_NODE %d", node->ID);
	FXWRITE;

	//if this isn't the last node return,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	current_save_node++;
	if (current_save_node!=(*num_nodes))
	{
		strcpy (buf, "MESSIAH_NODE_END");
		FXWRITE;
		return NULL;
	}


	//### This is the last node.  We're going to write an embedded scene file ###


	//create the embedded scene buffer................................
	if (!(m_buff = messiahSceneFileCreateBuff()))
	{
		ms->error("Couldn't save messiah's embedded scnene data!",NULL);
		return NULL;
	}

	strcpy (buf, "MESSIAH_FILE_BEGIN_BLOCK");
	FXWRITE;

	{
		//### Because LW's file format is ASCII, we need to explicity write each line
		//### of the embedded data.  Before we do this, we must strip the returns.

		messiahSceneFileFilterBuff(&m_buff, "\r");


		//write the lines,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		len = strlen(m_buff);

		buff1 = m_buff;

		while(buff2 = strchr(buff1, '\n'))
		{
			buff2[0] = '\0';

			if (len = strlen(buff1))
				sState->write(sState->writeData, buff1, len); //write the line
			else
				sState->write(sState->writeData, " ", 1); //if the line has no length, we must should at least write a space

			buff1 = &buff2[1];
		}

		//always clear the buffer after writing is done.
		messiahSceneFileClearBuff(m_buff);
	}

	strcpy (buf, "MESSIAH_FILE_END_BLOCK");
	FXWRITE;

	return (NULL);
}
	
//---------------------------------------------------------------------------
//	Name:	NodeFlags
//	Purpose:Set flags for this node
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static unsigned int)
NodeFlags (
	FX_NodeLog			*node)
{
	return LWDMF_BEFOREBONES;
}

//---------------------------------------------------------------------------
//	Name:	NodeFlagsMotion
//	Purpose:Set flags for this node
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static unsigned int)
NodeFlagsMotion (
	FX_NodeLog			*node)
{
	return 0;
}

//---------------------------------------------------------------------------
//	Name:	NodeNewTime
//	Purpose:Compute info based on a time change
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static LWError)
NodeNewTime (
	FX_NodeLog		*node,
	LWFrame			 f,
	LWTime			 t)
{
	LW_NodeData		*ndata=node->data;

	if (!ndata) return NULL;
/*
	//This may only be necessary to do for older versions of LightWave
	//because it doesn't tell you during Create if we're being loaded.
	//NewTime is only called after loading is done, so if we are loading
	//the object pointer will have been filled in.  If we're not loading
	//it will be NULL.
	if (!node->obj)
	{
		LWItemID		*element = (LWItemID*)node->element;
		char *name = (char*)(*it->name)(*element);
		node->obj = nodeObjectCreate(name, 0, 0);
		hostObjectGeometryCopy(node, 1);
	}
*/

	ndata->current_point = 0;

	loading_scene = 0;
	messiah_scene_saved = 0;
	load_counter = 0;

	messiahCheckTime((t * si->framesPerSecond), 1);

	return (NULL);
}

//---------------------------------------------------------------------------
//	Name:	NodeDisplacementEvaluate
//	Purpose:Compute new point positions
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static void)
NodeDisplacementEvaluate (
	FX_NodeLog				*node,
	LWDisplacementAccess	*sa)
{
	LW_NodeData		*ndata=node->data;
	double			point[3] = {0.0, 0.0, 0.0};

	if (!ndata) return;
	if (!node->obj) return;
	if (ndata->current_point>=ndata->num_points) return;

	nodeObjectGetPoint(node->obj, ndata->current_point, &point[0], &point[1], &point[2]);

	sa->source[0] += (point[0] - sa->oPos[0]);
	sa->source[1] += (point[1] - sa->oPos[1]);
	sa->source[2] += (point[2] - sa->oPos[2]);

	ndata->current_point++;

	return;
}

//---------------------------------------------------------------------------
//	Name:	NodeMotionEvaluate
//	Purpose:Compute new motion transformation
//	Callers:Lightwave
//---------------------------------------------------------------------------
	XCALL_(static void)
NodeMotionEvaluate (
	FX_NodeLog					*node,
	const LWItemMotionAccess	*im)
{
	double		pos[9];

	NodeNewTime (node,
		im->frame,
		im->time);

	if (node->obj)
	{
		(*im->getParam)(LWIP_POSITION, im->time, pos);
		(*im->getParam)(LWIP_ROTATION, im->time, &pos[3]);
		(*im->getParam)(LWIP_SCALING, im->time, &pos[6]);

		nodeObjectGetWorldPos(node->obj, pos, *messiah_current_frame);
		nodeObjectGetWorldRot(node->obj, &pos[3], *messiah_current_frame);
		nodeObjectGetWorldScale(node->obj, &pos[6], *messiah_current_frame);

		(*im->setParam)(LWIP_POSITION, pos);
		(*im->setParam)(LWIP_ROTATION, &pos[3]);
		(*im->setParam)(LWIP_SCALING, &pos[6]);
	}

	return;
}

//---------------------------------------------------------------------------
//	Name:	NodeDescln
//	Purpose:Provide description information for this node
//	Callers:Lightwave
//---------------------------------------------------------------------------
const char * NodeDescln(FX_NodeLog *node)
{
	static char txt[255];

	//Dan 5-23-02
	if(!cApplication_Name)
		MESSIAH_GET(cApplication_Name); //Dan 

	sprintf(txt, "%s Node:: %d of %d  %s", cApplication_Name, node->ID+1, counter,
		((*messiah_Activated)&MESSIAH_ACTIVATED_SYSTEM)?"Data Allocated":"No messiah Data");

//	sprintf(txt, "%s Node:: %d of %d  %s", "messiah:studio", node->ID+1, counter,
//		((*messiah_Activated)&MESSIAH_ACTIVATED_SYSTEM)?"Data Allocated":"No messiah Data");
	return txt;
}

//---------------------------------------------------------------------------
//	Name:	NodeUseItems
//	Purpose:Provide an array of used object IDs
//	Callers:Lightwave
//---------------------------------------------------------------------------
const LWItemID* NodeUseItems(FX_NodeLog *node)
{
   static LWItemID	ID[ 2 ] = { NULL, NULL };
   LWItemID			*element = (LWItemID*)node->element;

   ID[ 0 ] = *element;

	return ID;
}

//---------------------------------------------------------------------------
//	Name:	NodeChangeID
//	Purpose:Change an object's ID
//	Callers:Lightwave
//---------------------------------------------------------------------------
static void NodeChangeID (FX_NodeLog *node, const LWItemID *ID)
{
   int		i = 0;
   LWItemID *element = (LWItemID*)node->element;

   while ( ID[ i ] )
   {
      if ( ID[ i ] == (*element) )
	  {
         *element = ID[ i+1 ];
         node->element = element;

		 //This is how LW tells you that the geometry has changed... quirky.
		if (ID[i]==ID[i+1])
		{
			nodeObjectRebuild(node->obj, 1);
			messiahUpdate(0);
		}
        return;
      }
      else
	  {
		  if (!ID[i+1]) return;

         i += 2;
	  }
   }

	return;
}







/*

	MUST ACTIVATE THIS CODE.  THIS IS FOR THE MASTER MODULE THAT WILL MANAGE THE
	MESSIAH CONNECTION.  IT WILL BE IMPORTANT FOR DETERMINING LOAD AND OTHER STUFF.
*/

int		no_data_to_pass=0;

XCALL_(LWInstance) MasterCreate(void *priv,void *context,LWError *err)
{
	//Start the master module,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	MasterStart(); 

    return(&no_data_to_pass);
}

XCALL_(void) MasterDestroy(LWInstance inst)
{
	MasterShutdown();

	return;
}

XCALL_(LWError) MasterCopy(LWInstance d1,LWInstance d2)
{
    return(NULL);
}

XCALL_(LWError) MasterLoad(LWInstance inst,const LWLoadState *lState)
{
	//Since masters load first, we can set a flag to tell us when the scene is loading.
//	loading_scene = 1; //no longer necessary here, there is a new PreLoadScene message where this is set

    return(NULL);
}

XCALL_(LWError) MasterSave(LWInstance inst,const LWSaveState *sState)
{
	char		buf[4096]; //4k buffer

	strcpy (buf, "MESSIAH_MASTER");
	FXWRITE;
	return(NULL);
}

XCALL_(unsigned int) MasterFlags(LWInstance inst)
{
    return(LWMAST_LAYOUT);
}

XCALL_(const char *) MasterDescribe(LWInstance inst)
{
	//Dan 5-23-02
	if(!cApplication_Name)
		MESSIAH_GET(cApplication_Name); //Dan 

	return(cApplication_Name);
//    return("messiah:studio");
}


#define LWCOMMAND(c)	(!strcmp(c, command))

XCALL_( static double ) MasterEvent( LWInstance inst, const LWMasterAccess *ma )
{
	char	command[1024], *buff = ma->eventData;

	if (ma->eventCode!=LWEVNT_COMMAND)
	{
		return 0.0;
	}

	if (!buff) return 0.0;

	sscanf(buff, "%s", command);

	//Unfortunately, all of these messages are post.  LW doesn't issue
	//any kind of pre messaging, so we'll need to take care with the nodes.
	//We currently only use this to set flags, but it can be expanded to
	//support many of LW's features.

/*
	if (LWCOMMAND("LoadScene")||LWCOMMAND("LoadFromScene"))
	{
		current_save_node = 0;
		messiah_scene_saved = 0;
		load_counter = 0;

		if (loading_scene)
		{
			loading_scene = 0;
			messiahUpdate(0);
		}
	}
*/
	if (LWCOMMAND("PreLoadScene"))
	{
		current_save_node = 0;
		messiah_scene_saved = 0;
		load_counter = 0;

		loading_scene = 1;

//		if (loading_scene)
//		{
//			loading_scene = 0;
//			messiahUpdate(0);
//		}
	}
	else if (LWCOMMAND("SaveScene")||LWCOMMAND("SaveSceneAs"))
	{
		loading_scene = 0;
		current_save_node = 0;
		messiah_scene_saved = 0;
		load_counter = 0;
	}
	else if (LWCOMMAND("ClearScene"))
	{
		loading_scene = 0;
		current_save_node = 0;
		messiah_scene_saved = 0;
		load_counter = 0;
	}
	else if (LWCOMMAND("FramesPerSecond"))
	{
	}
	else if (LWCOMMAND("LoadScene"))
	{
		//scene is finished loading here
		loading_scene = 0;
		messiahUpdate(0);

	}

	return 0.0;
}




//################################################################################
//LW Activation Servers (plugin modules
//################################################################################


//---------------------------------------------------------------------------
//	Name:	messiahDisplacer
//	Purpose:Provide the necessary function pointers
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static int) messiahDisplacer (
	long					version,
	GlobalFunc				*global,
	LWDisplacementHandler	*local,
	void					*serverData)
{

	if (!(( version==LWDISPLACEMENT_VERSION )||(version==4))) 
		return (AFUNC_BADVERSION);

	if (AFUNC_BADGLOBAL==GetLWGlobals(global)) return AFUNC_BADGLOBAL;

	local->inst->create  = NodeCreateDisplace;
	local->inst->destroy = NodeDestroy;
	local->inst->load    = NodeLoad;
	local->inst->save    = NodeSave;
	local->inst->copy    = NodeCopy;
	local->inst->descln	 = NodeDescln;

	local->item->useItems = NodeUseItems;
	local->item->changeID = NodeChangeID;

	local->rend->newTime  = NodeNewTime;

	local->evaluate = NodeDisplacementEvaluate;
	local->flags    = NodeFlags;

	return (AFUNC_OK);
}


//---------------------------------------------------------------------------
//	Name:	messiahMover
//	Purpose:Provide the necessary function pointers
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static int) messiahMover (
	long					version,
	GlobalFunc				*global,
	LWItemMotionHandler		*local,
	void					*serverData)
{

	if ( version != LWITEMMOTION_VERSION )
		return (AFUNC_BADVERSION);

	if (AFUNC_BADGLOBAL==GetLWGlobals(global)) return AFUNC_BADGLOBAL;

	local->inst->create  = NodeCreateMotion;
	local->inst->destroy = NodeDestroy;
	local->inst->load    = NodeLoad;
	local->inst->save    = NodeSave;
	local->inst->copy    = NodeCopy;
	local->inst->descln  = NodeDescln;

	local->item->useItems = NodeUseItems;
	local->item->changeID = NodeChangeID;

	local->evaluate = NodeMotionEvaluate;
	local->flags    = NodeFlagsMotion;


	return (AFUNC_OK);
}


//---------------------------------------------------------------------------
//	Name:	messiahLWInterface
//	Purpose:Call the messiahActivate interface function
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static int) messiahLWInterface (
	long			version,
	GlobalFunc		*global,
	FX_NodeLog		*node,
	void			*serverData)
{
	if (AFUNC_BADGLOBAL==GetLWGlobals(global)) return AFUNC_BADGLOBAL;

	//Start up the messiah manager=====================================================
	if (!messiahStart(HOST_APP_NAME, HOST_MODULE_NAME, hostGlobal, MESSIAH_HOSTAPI_VERSION))
	{
		(*ms->error)("Can't start messiah!", NULL);
		return AFUNC_BADGLOBAL;
	}

	messiahInterface();

	return (AFUNC_OK);
}


//---------------------------------------------------------------------------
//	Name:	messiahMaster
//	Purpose:Manage the connection... (in theory;})
//	Callers:Lightwave
//---------------------------------------------------------------------------
XCALL_(static int) messiahMaster(
	long              version,
	GlobalFunc        *global,
	LWMasterHandler   *local,
	void              *serverData)
{
    if(version != LWMASTER_VERSION)
        return(AFUNC_BADVERSION);

 	if (AFUNC_BADGLOBAL==GetLWGlobals(global)) return AFUNC_BADGLOBAL;

	local->inst->create     = MasterCreate;
	local->inst->destroy    = MasterDestroy;
	local->inst->load       = MasterLoad;
	local->inst->save       = MasterSave;
	local->inst->copy       = MasterCopy;
	local->inst->descln     = MasterDescribe;

	local->item->useItems = NULL;//NodeUseItems;
	local->item->changeID = NULL;//NodeChangeID;

	local->type           = LWMAST_LAYOUT;
	local->event            = MasterEvent;
	local->flags            = MasterFlags;

//	master_active = 1; //don't want to put this here - moved to the MasterCreate function

    return(AFUNC_OK);
}





//################################################################################
//LW Plugin Setup
//################################################################################



ServerRecord ServerDesc[] = 
{
	{ LWMASTER_HCLASS,			"messiah:master",	messiahMaster },
	{ LWDISPLACEMENT_HCLASS,	"messiah:displace",	messiahDisplacer },
	{ LWDISPLACEMENT_ICLASS,	"messiah:displace",	messiahLWInterface },
	{ LWITEMMOTION_HCLASS,		"messiah:xform",	messiahMover },
	{ LWITEMMOTION_ICLASS,		"messiah:xform",	messiahLWInterface },
	{ LWLAYOUTGENERIC_CLASS,	"messiah:interface",messiahLWInterface },
	{ NULL }
};

void * Startup (void)
{
 	if (!hostLoadmessiahHOST()) return NULL;

   return (void *) 4;
}

void Shutdown (void *serverData)
{
	if (messiahlib)
	{
		messiahShutdown();

		FreeLibrary(messiahlib);
		messiahlib = NULL;
	}
}

ModuleDescriptor _mod_descrip =
 {
    MOD_SYSSYNC,
    MOD_SYSVER,
    MOD_MACHINE,
    Startup,
    Shutdown,
    ServerDesc
 };


