// ============		messiahMayaManager.h			==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Class Name: messiahMayaManager
//
// ===============================================================
//
// Class Description:
//
// This class is a internal manager, neither Maya nor messiah uses this class.
// This class is also a Singleton and is only accessible through:
//
//		MAN::Inst()
//
// which returns a pointer to the only instance of this class.
//
// Pointers to all messiahMaya related nodes are stored on a list
// in this object by calling AddNode() and removed by calling 
// RemoveNode().
//
// SceneNode() sets the current messiahNode for a load operation.
// This node holds the stored fxs buffer. See messiahNode.h
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#ifndef MESSIAH_MAYA_MANAGER
#define MESSIAH_MAYA_MANAGER

#include <vector>

#include <maya/MObject.h>

class MPxNode;
class MTime;

class messiahMayaManager
{
public:

	static messiahMayaManager *			Inst();

	// Scene State Flags
	static int const SceneLoading;
	static int const SceneSaving;
	static int const SceneClearing;
	static int const SceneActiveMessiahSession;

	int				SceneState	();
	int				SceneState	( int flag, bool state );

	double			CurrentTime		() { return _time; }

	// Called from the cst/dst of the different nodes
	void			AddNode			(MPxNode *node);
	void			RemoveNode		(MPxNode *node);
	void			SceneNode		(MPxNode *node);
	
	// Scene Callbacks (passed to Maya in pluginMain.cpp)
	static void		OnLoadPre		(void *data);
	static void		OnLoadPost		(void *data);
	static void		OnSavePre		(void *data);
	static void		OnSavePost		(void *data);
	static void		OnClearPre		(void *data);
	static void		OnClearPost		(void *data);
	static void		OnTimeChange	(MTime &time, void *data);
	
protected:
	// instantiation can only be done through Inst()
	messiahMayaManager() : _sceneState(0) {}

private:
	// instance
	static messiahMayaManager *	_inst;

	std::vector<MPxNode *>		_nodes;
	int							_sceneState;
	double						_time;

	MPxNode	*					_sceneNode;

};

// shortcut
typedef messiahMayaManager MAN;

#endif

