#include "c4d_memory.h"
#include "c4d_basetag.h"
#include "c4d_baseobject.h"
#include "c4d_basedocument.h"
#include "c4d_basecontainer.h"
#include "c4d_basebitmap.h"
#include "c4d_raytrace.h"
#include "c4d_thread.h"
#include "c4d_general.h"

RenderData *AllocRenderData(void)
{						
	return C4DOS.Bd->RdAlloc();
}

BaseContainer RenderData::GetData(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseContainer();
	C4DOS.Bd->RdGetData(this,bc.Get());
	return bc;
}

void RenderData::SetData(const BaseContainer &bc)
{
	if (!bc.Get()) return;
	C4DOS.Bd->RdSetData(this,bc.Get());
}

BaseDocument *AllocBaseDocument(void)
{						
	return C4DOS.Bd->Alloc();
}

BaseContainer BaseDocument::GetData(LONG type)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseContainer();
	C4DOS.Bd->GetData(this,type,bc.Get());
	return bc;
}

void BaseDocument::SetData(LONG type, const BaseContainer &bc)
{
	if (!bc.Get()) return;
	C4DOS.Bd->SetData(this,type,bc.Get());
}

BaseObject *BaseDocument::GetFirstObject(void)
{
	return C4DOS.Bd->GetFirstObject(this);
}

BaseMaterial *BaseDocument::GetFirstMaterial(void)
{
	return C4DOS.Bd->GetFirstMaterial(this);
}

RenderData *BaseDocument::GetFirstRenderData(void)
{
	return C4DOS.Bd->GetFirstRenderData(this);
}

void BaseDocument::InsertRenderData(RenderData *rd, RenderData *pred)
{
	C4DOS.Bd->InsertRenderData(this,rd,pred);
}

void BaseDocument::InsertMaterial(BaseMaterial *mat, BaseMaterial *pred, Bool checknames)
{
	C4DOS.Bd->InsertMaterial(this,mat,pred,checknames);
}

void BaseDocument::InsertObject(BaseObject *op, BaseObject *parent, BaseObject *pred, Bool checknames)
{
	C4DOS.Bd->InsertObject(this,op,parent,pred,checknames);
}

RenderData *BaseDocument::GetActiveRenderData(void)
{
	return C4DOS.Bd->GetActiveRenderData(this);
}

BaseObject *BaseDocument::GetActiveObject(void)
{
	return C4DOS.Bd->GetActiveObject(this);
}

BaseMaterial *BaseDocument::GetActiveMaterial(void)
{
	return C4DOS.Bd->GetActiveMaterial(this);
}

TextureTag *BaseDocument::GetActiveTexTag(void)
{
	BaseTag *ta = GetActiveTag();
	if (ta && ta->GetType()==Ttexture) return (TextureTag*)ta;

	BaseObject *op = GetActiveObject(); 
	if (!op) return NULL;
	return (TextureTag*)op->GetTag(Ttexture);
}

BaseTag *BaseDocument::GetActiveTag(void)  
{
	return C4DOS.Bd->GetActiveTag(this,NULL);
}

void BaseDocument::SetActiveObject(BaseObject *op)
{
	C4DOS.Bd->SetActiveObject(this,op);
}

void BaseDocument::SetActiveTag(BaseObject *op, BaseTag *ta)	
{
	C4DOS.Bd->SetActiveTag(this,op,ta);
}

void BaseDocument::SetActiveMaterial(BaseMaterial	*mp)                  
{
	C4DOS.Bd->SetActiveMaterial(this,mp);
}

void BaseDocument::SetActiveRenderData(RenderData *rd)
{
	C4DOS.Bd->SetActiveRenderData(this,rd);
}

Bool BaseDocument::CopyTo(BaseDocument *dest, LONG flags)
{
	return C4DOS.Bd->CopyTo(this,dest,flags);
}

BaseDocument *BaseDocument::GetClone(LONG flags)
{
	BaseDocument *cpy = AllocBaseDocument();
	if (cpy && !CopyTo(cpy,flags)) FreeBaseDocument(cpy);
	return cpy;
}

void BaseDocument::Flush(void) 
{
	C4DOS.Bd->FlushAll(this);
}

BaseObject *BaseDocument::GetHighest(LONG type, Bool editor) 
{
	return C4DOS.Bd->GetHighest(this,type,editor);
}

BaseMaterial *BaseDocument::SearchMaterial(const String &str)
{
	if (!str.Get()) return NULL;
	return C4DOS.Bd->SearchMaterial(this,str.Get(),FALSE);
}

BaseObject *BaseDocument::SearchObject(const String &str)	
{
	if (!str.Get()) return NULL;
	return C4DOS.Bd->SearchObject(this,str.Get(),FALSE);
}

BaseMaterial *BaseDocument::SearchMaterialInc(const String &str)
{
	if (!str.Get()) return NULL;
	return C4DOS.Bd->SearchMaterial(this,str.Get(),TRUE);
}

BaseObject *BaseDocument::SearchObjectInc(const String &str)	
{
	if (!str.Get()) return NULL;
	return C4DOS.Bd->SearchObject(this,str.Get(),TRUE);
}

Bool BaseDocument::GetChanged(void)
{
	BaseContainer bc;
	if (!bc.Get()) return FALSE;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetBool(DOCUMENT_CHANGED);
}

void BaseDocument::SetChanged(void)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetBool(DOCUMENT_CHANGED,TRUE);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetName(const Filename &fn)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetFilename(DOCUMENT_NAME,fn);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetPath(const Filename &fn)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetFilename(DOCUMENT_PATH,fn);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

Filename BaseDocument::GetName(void)
{
	BaseContainer bc;
	if (!bc.Get()) return Filename();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetFilename(DOCUMENT_NAME);
}

Filename BaseDocument::GetPath(void)
{
	BaseContainer bc;
	if (!bc.Get()) return Filename();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetFilename(DOCUMENT_PATH);
}

Real BaseDocument::GetLOD(void)
{
	BaseContainer bc;
	if (!bc.Get()) return 1.0;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetReal(DOCUMENT_LOD);
}

LONG BaseDocument::GetFps(void)
{
	BaseContainer bc;
	if (!bc.Get()) return 30;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetLong(DOCUMENT_FPS);
}

BaseTime BaseDocument::GetMinTime(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseTime();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetTime(DOCUMENT_MINTIME);
}

BaseTime BaseDocument::GetMaxTime(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseTime();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetTime(DOCUMENT_MAXTIME);
}

BaseTime BaseDocument::GetLoopMinTime(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseTime();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetTime(DOCUMENT_LOOPMINTIME);
}

BaseTime BaseDocument::GetLoopMaxTime(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseTime();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetTime(DOCUMENT_LOOPMAXTIME);
}

Bool BaseDocument::GetRenderLod(void)
{
	BaseContainer bc;
	if (!bc.Get()) return FALSE;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetBool(DOCUMENT_RENDERLOD);
}

void BaseDocument::SetLOD(Real lod)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetReal(DOCUMENT_LOD,lod);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetFps(LONG fps)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetLong(DOCUMENT_FPS,fps);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetMinTime(const BaseTime &t)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetTime(DOCUMENT_MINTIME,t);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetMaxTime(const BaseTime &t)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetTime(DOCUMENT_MAXTIME,t);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetLoopMinTime(const BaseTime &t)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetTime(DOCUMENT_LOOPMINTIME,t);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetLoopMaxTime(const BaseTime &t)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetTime(DOCUMENT_LOOPMAXTIME,t);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetRenderLod(Bool lod)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetBool(DOCUMENT_RENDERLOD,lod);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

BaseTime BaseDocument::GetTime(void)
{
	BaseContainer bc;
	if (!bc.Get()) return BaseTime();
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetTime(DOCUMENT_TIME);
}

LONG BaseDocument::GetMode(void)
{
	BaseContainer bc;
	if (!bc.Get()) return 0;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetLong(DOCUMENT_MODE);
}

LONG BaseDocument::GetAction(void)
{
	BaseContainer bc;
	if (!bc.Get()) return 0;
	C4DOS.Bd->GetData(this,SETTINGS_GENERAL,bc.Get());
	return bc.GetLong(DOCUMENT_ACTION);
}

Bool BaseDocument::IsEditMode(void)
{
	LONG mode=GetMode();
	return mode==Mpoints || mode==Mpolygons; 
}

void BaseDocument::SetTime(const BaseTime &t)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetTime(DOCUMENT_TIME,t);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetMode(LONG m)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetLong(DOCUMENT_MODE,m);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void BaseDocument::SetAction(LONG a)
{
	BaseContainer bc;
	if (!bc.Get()) return;
	bc.SetLong(DOCUMENT_ACTION,a);
	C4DOS.Bd->SetData(this,SETTINGS_GENERAL,bc.Get());
}

void InsertBaseDocument(BaseDocument *doc)
{
	C4DOS.Ge->InsertBaseDocument(doc);
}

void SetActiveDocument(BaseDocument *doc)
{
	C4DOS.Ge->SetActiveDocument(doc);
}

BaseDocument *GetActiveDocument(void)
{
	return C4DOS.Ge->GetActiveDocument();
}

BaseDocument *GetFirstDocument(void)
{
	return C4DOS.Ge->GetFirstDocument();
}

void KillDocument(BaseDocument *doc)
{
	C4DOS.Ge->KillDocument(doc);
}

Bool LoadFile(const Filename &name)
{
	if (!name.Get()) return FALSE;
	return C4DOS.Ge->LoadFile(name.Get());
}

BaseDocument *LoadDocument(const Filename &name, Bool show_error)
{
	if (!name.Get()) return NULL;
	BaseDocument *doc=AllocBaseDocument();
	if (!doc) return NULL;
	if (!C4DOS.Ge->LoadDocument(doc,name.Get(),show_error))
		blDelete(doc);
	return doc;
}

Bool SaveDocument(BaseDocument *doc, const Filename &name, Bool show_error, LONG format)
{
	if (!name.Get()) return FALSE;
	doc->SetPath(name.GetDirectory());
	doc->SetName(name.GetFile());
	return C4DOS.Ge->SaveDocument(doc,name.Get(),show_error,format);
}

LONG RenderDocument(BaseDocument *doc, ProgressHook *pr, void *private_data, BaseBitmap *bmp, 
									  const BaseContainer &rdata, Bool external, Bool show_errors, Thread *th, LONG v_left, LONG v_top, LONG v_right, LONG v_bottom)
{
	if (!rdata.Get()) return RAY_NOMEM;
	return C4DOS.Ge->RenderDocument(doc,pr,private_data,bmp,rdata.Get(),external,show_errors,th?th->Get():NULL,v_left,v_top,v_right,v_bottom);
}

Bool BaseDocument::StartUndo(void)
{
	return C4DOS.Bd->StartUndo(this);
}

Bool BaseDocument::EndUndo(void)
{
	return C4DOS.Bd->EndUndo(this);
}

Bool BaseDocument::AddUndo(LONG type, void *data)
{
	return C4DOS.Bd->AddUndo(this,type,data);
}

Bool BaseDocument::DoUndo(void)
{
	return C4DOS.Bd->DoUndo(this);
}

Bool BaseDocument::DoRedo(void)
{
	return C4DOS.Bd->DoRedo(this);
}

void BaseDocument::AnimateDocument(BaseThread *bt,Bool doexpressions,Bool doanim)
{
	C4DOS.Bd->AnimateDocument(this,bt,doexpressions,doanim);
}

void BaseDocument::AnimateObject(BaseObject *op, const BaseTime &time, LONG flags)
{
	C4DOS.Bd->AnimateObject(this,op,time,flags);
}

BaseDocument *BaseDocument::Polygonize(void)
{
	BaseDocument	*dst=AllocBaseDocument(); if (!dst) return NULL;
	BaseObject		*nx=NULL,*res=NULL,*op=NULL;

	if (!CopyTo(dst)) goto Error;

	for (op=dst->GetFirstObject(); op; op=nx)
	{
		nx=op->next();
		res=(BaseObject*)SendModelingCommand(MCOMMAND_CURRENTSTATETOOBJECT,this,op,NULL,MODIFY_ALL);
		if (!res) goto Error;
		res->InsertBefore(op);
		blDelete(op);
	}

	return dst;

Error:
	blDelete(dst);
	return NULL;
}

BaseDraw *BaseDocument::GetActiveBaseDraw(void)
{
	return C4DOS.Bd->GetActiveBaseDraw(this);
}

BaseDraw *BaseDocument::GetRenderBaseDraw(void)
{
	return C4DOS.Bd->GetRenderBaseDraw(this);
}

BaseDraw *BaseDocument::GetBaseDraw(LONG num)
{
	return C4DOS.Bd->GetBaseDraw(this,num);
}

LONG BaseDocument::GetSplinePlane(void)
{
	return C4DOS.Bd->GetSplinePlane(this);
}

void *HAlloc(void *main) { return ((Hierarchy*)main)->Alloc(); }
void HFree(void *main, void *data) { ((Hierarchy*)main)->Free(data); }
void HCopyTo(void *main, void *src, void *dst) { ((Hierarchy*)main)->CopyTo(src,dst); }
Bool HDo(void *main, void *data, BaseObject *op, const Matrix &mg, Bool controlobject) { return ((Hierarchy*)main)->Do(data,op,mg,controlobject); }

Bool Hierarchy::Run(BaseDocument *t_doc, Bool spheres, Real lod, LONG flags, void *startdata, BaseThread *t_bt)
{
	doc=t_doc;
	bt =t_bt;
	return C4DOS.Bd->RunHierarchy(this,doc,spheres,lod,flags,startdata,bt,HAlloc,HFree,HCopyTo,HDo);
}
