#ifndef __C4DBASELIST_H
#define __C4DBASELIST_H

#include "ge_math.h"
#include "c4d_string.h"

struct VariableChanged
{
	VariableChanged(void);

	LONG old_cnt,new_cnt,*map;
	Bool safety;
};

#define MSG_POINTS_CHANGED		 1
#define MSG_POLYGONS_CHANGED	 2
#define MSG_UPDATE						 5
#define MSG_SMALLUPDATE				 6
#define MSG_CHANGE						 7
#define MSG_BASECONTAINER      9
#define MSG_SEGMENTS_CHANGED  10
#define MSG_DOCUMENT_CLONED		11

class Marker
{
	friend class BaseList2D;
	friend Marker GeGetNewMarker(void);
	friend Bool operator == (const Marker &m1, const Marker &m2);
	friend Bool operator != (const Marker &m1, const Marker &m2);
	
	protected:
		ULONG L1,L2;
};

Marker GeGetNewMarker(void);

class BaseList2D
{
	private:
		BaseList2D(void);
		~BaseList2D(void);
	public:
 		LONG GetType(void) { return C4DOS.Bl->GetType(this); }
 		LONG GetDiskType(void);     

		Marker GetMarker (void); 

		BaseList2D *next(void) { return C4DOS.Bl->Next(this); }
		BaseList2D *pred(void) { return C4DOS.Bl->Pred(this);	}

		void InsertBefore(BaseList2D *bl); 
		void InsertAfter (BaseList2D *bl); 
		void Remove(void); 

		void SetBit   (LONG mask) { C4DOS.Bl->SetAllBits(this,C4DOS.Bl->GetAllBits(this)|mask); }
		Bool GetBit   (LONG mask) { return (C4DOS.Bl->GetAllBits(this)&mask)==mask;	}
		void DelBit   (LONG mask) { C4DOS.Bl->SetAllBits(this,C4DOS.Bl->GetAllBits(this) & ~mask); }
		void ToggleBit(LONG mask);
		LONG GetAllBits(void) { return C4DOS.Bl->GetAllBits(this);	}
		void SetAllBits(LONG bits) { C4DOS.Bl->SetAllBits(this,bits); }

		LONG GetColor  (void);
		void SetColor  (LONG c);

		Bool Message(LONG type, void *data=NULL);  
};

class BaseList4D : public BaseList2D
{
	private:
		BaseList4D(void);
		~BaseList4D(void);
	public:
		BaseList4D *next(void) { return (BaseList4D*)C4DOS.Bl->Next(this); }
		BaseList4D *pred(void) { return (BaseList4D*)C4DOS.Bl->Pred(this); }
		BaseList4D *up  (void) { return (BaseList4D*)C4DOS.Bl->Up(this); }
		BaseList4D *down(void) { return (BaseList4D*)C4DOS.Bl->Down(this); }
		BaseList4D *downlast(void) { return (BaseList4D*)C4DOS.Bl->DownLast(this); }

		void InsertUnder    (BaseList4D *bl);
		void InsertUnderLast(BaseList4D *bl);
};

#define blDelete(v) { if (v) C4DOS.Bl->Free(v); v=NULL;}

#endif
