#ifndef __C4DBASETAG_H
#define __C4DBASETAG_H

#include "c4d_baselist.h"
#include "operatingsystem.h"
#include "ge_vector.h"

class BaseObject;
class BaseSelect;

class BaseTag : public BaseList2D
{
	private:
		BaseTag(void);
		~BaseTag(void);
	public:
		BaseTag 			*GetClone(LONG flags=0);
		BaseObject		*GetObject(void) { return (BaseObject*)C4DOS.Bl->GetMain(this); }

		Bool    			CopyTo(BaseTag *dest, LONG flags=0);
		LONG 					GetTagInfo(void);
		Bool 					Edit(void); 
		void					SetData(const BaseContainer &bc);
		BaseContainer GetData(void);
		void					GetIcon(IconData *dat);

		BaseTag *next(void) { return (BaseTag*)C4DOS.Bl->Next(this); }
		BaseTag *pred(void) { return (BaseTag*)C4DOS.Bl->Pred(this); }
};

class VariableTag : public BaseTag
{
	private:
		VariableTag(void);
		~VariableTag(void);
	public:
		LONG 				GetDataCount	(void);
		LONG 				GetDataSize		(void);
		void 			 *GetDataAddress(void);
};

class SelectionTag : public BaseTag
{
	private:
		SelectionTag(void);
		~SelectionTag(void);
	public:
		BaseSelect *GetBaseSelect(void);
};

struct UVWStruct
{
	UVWStruct(void) {}
	UVWStruct(const Vector &t_a, const Vector &t_b, const Vector &t_c, const Vector &t_d) { a=t_a; b=t_b; c=t_c; d=t_d; }
	UVWStruct(const Vector &t_a, const Vector &t_b, const Vector &t_c) { a=t_a; b=t_b; c=t_c; }

	Vector a,b,c,d;
};

class UVWTag : public VariableTag
{
	private:
		UVWTag(void);
		~UVWTag(void);
	public:
		UVWStruct Get(LONG i) { UVWStruct us; C4DOS.Tg->UvGet(this,i,&us); return us; }
		void Set(LONG i, const UVWStruct &s) { C4DOS.Tg->UvSet(this,i,(UVWStruct*) &s); }
		void Cpy(LONG dst, UVWTag *srctag, LONG src) { C4DOS.Tg->UvCpy(this,dst,srctag,src); }
};

class TextureTag : public BaseTag
{
	private:
		TextureTag(void);
		~TextureTag(void);
	public:
		Vector GetPos(void);
		Vector GetScale(void);
		Vector GetRot(void);
		Matrix GetMl(void);
		Matrix GetMln(void);
		void SetPos(const Vector &v);
		void SetScale(const Vector &v);
		void SetRot(const Vector &v);
		void SetMl(const Matrix &m);

		BaseMaterial *GetMaterial(void);
		void SetMaterial(BaseMaterial *mat);

		BaseMaterial *GetSecondMaterial(Real *fade);
		void SetSecondMaterial(BaseMaterial *mat, Real fade);

		BaseMaterial *GetInterpolatedMaterial(BaseDocument *doc);
};

class StickTextureTag : public VariableTag
{
	private:
		StickTextureTag(void);
		~StickTextureTag(void);
	public:
		Bool Record(BaseObject *op, Bool always);
};

class PluginTag : public BaseTag
{
	private:
		PluginTag(void);
		~PluginTag(void);
	public:
		LONG GetPluginID(void);
		void *GetPluginData(void);
		void SetPluginData(void *mem);
};

BaseTag *AllocTag(LONG type, LONG subtype, LONG count);
#define FreeTag(v) { if (v) C4DOS.Bl->Free(v); v=NULL;}

#endif
