/////////////////////////////////////////////////////////////
// CINEMA SDK : RENDERING																	 //
/////////////////////////////////////////////////////////////
// VERSION    : CINEMA 4D V6 AMBER												 //
/////////////////////////////////////////////////////////////
// (c) 1989-2001 MAXON Computer GmbH, all rights reserved	 //
/////////////////////////////////////////////////////////////

// video post classes

#ifndef __C4D_VIDEOPOST_H
#define __C4D_VIDEOPOST_H

#include "ge_vector.h"
#include "c4d_basetime.h"

struct PLUGIN_DATA;
struct VolumeData;
struct VideoPostStruct;
struct RayFragment;
struct Multipass;
struct PixelFragment;
class  BaseDocument;
class  BaseThread;
class  BaseDraw;
class  BaseDrawHelp;
class  HyperFile;
class  Render;
class  _BaseContainer;
class  CDialog;

struct BaseVideoPostStruct
{
	// C4D version number
	LONG					version;	

	// VP_xxx type
	LONG					vp;

	// TRUE if it is an opener, FALSE if it is a closer
	Bool					open;

	// current time
	BaseTime			time;			

	// render frame rate (this may be different to doc->GetFps())
	LONG					fps;      

	// pointer to LONG containing RAY_xxx states
	LONG					*error;

	// pointer to VolumeData (see c4d_shaderplugin.h); may be NULL
	VolumeData		*vd;	

	// document
	BaseDocument	*doc;

	// thread; use thread->TestBreak() to check for a user break
	BaseThread		*thread;

	// current subframe and number of subframes
	LONG					subframe,subframe_cnt;

	// current field and number of fields
	LONG					field,field_cnt;

	// rendering in editor
	Bool					editor_render;

	// render instance
	Render				*render;
};

#define VIDEOPOST_HIDDEN			(1<<0)	// invisible effects cannot be accessed by the user and are always called
#define VIDEOPOST_RAYTRACING	(1<<1)	// video post needs raytracing
#define VIDEOPOST_REFRESH			(1<<2)	// external update screen
#define VIDEOPOST_MULTIPLE		(1<<3)	// allow multiple effects of this type

#define VP_FRAMESEQUENCE	1 // called when the renderer prepares to render a sequence of images
#define VP_FRAME					2 // called when the renderer prepares to render a frame
#define VP_SUBFRAME				3 // called when the renderer prepares to render a subframe
#define VP_RENDER					4 // called before the rendering process starts and before shaders are initialized
#define VP_INNER					5 // called immediately before rendering starts

// common buffers
#define VPBUFFER_RGBA								 1 
#define VPBUFFER_AMBIENT						 2 
#define VPBUFFER_DIFFUSE						 3 
#define VPBUFFER_SPECULAR						 4 
#define VPBUFFER_SHADOW							 5 
#define VPBUFFER_REFLECTION					 6
#define VPBUFFER_TRANSPARENCY				 7
#define VPBUFFER_RADIOSITY					 8
#define VPBUFFER_CAUSTICS						 9
#define VPBUFFER_ATMOSPHERE					10
#define VPBUFFER_ATMOSPHERE_MUL			11

#define VPBUFFER_IMAGEBUILDING_PASS	32

// special buffers
#define VPBUFFER_DEPTH							100
#define VPBUFFER_MAT_COLOR					101
#define VPBUFFER_MAT_DIFFUSION			102
#define VPBUFFER_MAT_LUMINANCE			103
#define VPBUFFER_MAT_TRANSPARENCY		104
#define VPBUFFER_MAT_REFLECTION			105
#define VPBUFFER_MAT_ENVIRONMENT		106
#define VPBUFFER_MAT_SPECULAR				107
#define VPBUFFER_MAT_SPECULARCOLOR	108
#define VPBUFFER_ILLUMINATION				109
#define VPBUFFER_OBJECTBUFFER				110

#define VPBUFFER_BLEND						 1000
#define VPBUFFER_LIGHTBLEND				 1001 // private ID
#define VPBUFFER_ALPHA             1002 // for sdk request only 

struct PixelPost
{
#ifdef __API_INTERN__
	RayFragment **frag;
#else
	PixelFragment **frag;
#endif
	VolumeData	*vd;
	Real				*col;
	Multipass		*mp;
	Bool				aa,valid_line;
	LONG				comp;
	LONG				cpu_num;
	LONG				left,right,line;
};

struct VP_DATA
{
	_BaseContainer *settings;	// pointer to settings 
	void *render;							// pointer to render structures
};

#define VPEFFECT_DEPTHOFFIELD						1000
#define VPEFFECT_LENSEFFECTS						1001
#define VPEFFECT_GLOW										1002
#define VPEFFECT_COLORCORRECTION				1003
#define VPEFFECT_COMIC									1004
#define VPEFFECT_SCENEMOTIONBLUR				1005
#define VPEFFECT_OBJECTMOTIONBLUR				1006
#define VPEFFECT_FILTERSOFT							1007
#define VPEFFECT_FILTERSHARPEN					1008
#define VPEFFECT_FILTERMEDIAN						1009
#define VPEFFECT_EXTERNAL								1010

#define VPPRIORITY_GLOW									 500
#define VPPRIORITY_OBJECTMOTIONBLUR			 400
#define VPPRIORITY_DEPTHOFFIELD					 300
#define VPPRIORITY_LENSEFFECTS					 200
#define VPPRIORITY_FILTERSOFT						 100
#define VPPRIORITY_FILTERSHARPEN				 100
#define VPPRIORITY_FILTERMEDIAN					 100

#define VPPRIORITY_COMIC								 600
#define VPPRIORITY_COLORCORRECTION			 500
#define VPPRIORITY_SCENEMOTIONBLUR			 100
#define VPPRIORITY_EXTERNAL							 100

struct VIDEOPOST
{
	// disk level
	LONG level;

	// VIDEOPOST_xxx bits
	LONG info;

	void	*unused[6];

	// message routine to receive arbitrary data
	Bool			(*Message         )(VP_DATA *s, LONG type, void *data);

	// allows the VideoPost to draw elements in the editor
	Bool			(*Draw						)(VP_DATA *s, BaseDraw *bd, BaseDrawHelp *bh);

	// main call, returns RAY_xxx states
	LONG			(*Execute					)(VP_DATA *s, VideoPostStruct *vps);

	void			(*AllocateBuffers	)(VP_DATA *s, Render *render);

	// routines to maintain settings; must be NULL if VIDEOPOST_HIDDEN is set
	Bool			(*AllocSettings		)(VP_DATA *s);
	
	void			(*ExecuteLine			)(VP_DATA *s, PixelPost *pp);
	void			(*ExecutePixel    )(VP_DATA *s, PixelPost *pp, LONG x, LONG subx, LONG suby);

	CDialog*	(*AllocDialog			)(_BaseContainer *bc, CDialog *cd_parent, LONG dlg_id);
	void			(*FreeDialog      )(CDialog *cd);

	LONG	priority;
};

#endif
