#ifndef __SYSMATH_H
#define __SYSMATH_H

#include <fp.h>

struct Matrix;
struct Vector;

#ifndef NULL
	#define NULL 0
#endif

#ifndef NIL
	#define NIL 0
#endif

#ifndef TRUE
	#define TRUE	1
#endif

#ifndef FALSE
	#define FALSE	0
#endif

typedef void*								Hndl;
typedef int                 Bool;

typedef char								CHAR;
typedef unsigned char				UCHAR;
typedef short int	          WORD;
typedef unsigned short int	UWORD;
typedef long 								LONG;
typedef unsigned long				ULONG;
typedef int									INT;
typedef unsigned int 				UINT;
typedef float								Real;
typedef float								SReal;
typedef double							LReal;

#define pi					3.1415926535897932384626433832795
#define pi2					6.283185307179586476925286766559
#define pi05				1.5707963267948966192313216916398
#define sqrt2				1.4142135623730950488016887242097
#define onesqrt2		0.70710678118654752440084436210485

#define MAXREAL			9.0e18
#define MAXLONG			2147483647L
#define MAXULONG		4294967295L
#define MAXWORD			32767
#define MAXUWORD		65535

#define MMin(a,b)		(((a)<(b))?(a):(b))
#define MMax(a,b)		(((a)>(b))?(a):(b))
#define Cut(a,b,c)	(MMax(MMin((a),(c)),(b)))

#define Abs(a) 			(((a)<0)?-(a):(a))
#define Mod(a,b)		((a)<0?((a)-(((a)-(b)+1)/(b))*(b)):(a)%(b))
#define FMod(r,s)		fmod(r,s)
#define Sin(r) 			sin(r)
#define Cos(r) 			cos(r)
#define Tan(r) 			tan(r)
#define ATan(r) 		atan(r)
#define Exp(r)			exp(r)
#define Ln(r) 			log(r)
#define Ln10(r) 		log10(r)
#define Sqrt(r)			sqrt(r)
#define Floor(r)		floor(r)
#define Ceil(r)			ceil(r)
#define Pow(r,s)		pow((r),(s))
#define Cosh(r)			cosh(r)
#define Sinh(r)			sinh(r)
#define Tanh(r)			tanh(r)

inline void SinCos(Real w,Real &sn,Real &cn)
{
	sn=Sin(w);
	cn=Cos(w);
}//SinCos

inline Real ASin(Real val)
{
	if(val>=1.0) return pi05;
	else if(val<=-1.0) return -pi05;
	return asin((double)val);
}//ASin

inline Real ACos(Real val)
{
	if(val>=1.0) return 0.0;
	else if(val<=-1.0) return pi;
	return acos((double)val);
}//ACos

inline Real Rad(Real r){return ((r)*pi/180.0);}
inline Real Deg(Real r){return ((r)*180.0/pi);}

inline LONG SAFELONG(Real x)
{
	if (x<=-Real(MAXLONG)) return -MAXLONG;
	else if (x>=Real(MAXLONG)) return MAXLONG;
	return LONG(x);
}

#endif
