// Starts the plugin registration

#include "c4d_general.h"
#include "c4d_resource.h"
#include "c4d_thread.h"

#include "messiahDLL.h"
#include "ConfigData.h"

#include "Diagnostic.h"
#include <direct.h>

// Forward declarations
Bool RegisterMessiahConnection();
void FreeMessiahConnection();

Bool RegisterMessiahTag();
void FreeMessiahTag();

Bool CreateManager();
void FreeManager();

#define FX_HOST_MODULE_NAME		"messiahC4D.cdl"
#define FX_HOST_APP_NAME		  "Cinema4D"

static Semaphore* s = NULL;
static char pluginDir[2048];
static char cwd[2048];

Bool C4D_PlStart(void *p1, void *p2)
{
	s = AllocSemaphore();
//	s->Lock();

	BeginDiagnostic();

	Filename plugPath;
	Filename startupPath;
	String tmpstr;
	LONG len;
	char buff[2048];

	// get current working directory
	_getcwd(cwd, 2048);

	startupPath = GeGetStartupPath();
	plugPath = GeGetPluginPath();

	tmpstr = startupPath.GetString();
	len= tmpstr.GetCStringLen() + 1;
	tmpstr.GetCString(buff, len);

	AppendDiagnostic("STRPTH ");
	AppendDiagnostic(buff);
	AppendDiagnostic("\n");

	tmpstr = plugPath.GetString();
	len= tmpstr.GetCStringLen() + 1;
	tmpstr.GetCString(pluginDir, len);

	AppendDiagnostic("PLGPTH ");
	AppendDiagnostic(pluginDir);
	AppendDiagnostic("\n");


  GePrint("C4D_PlStart");

	// Initialize resources
	AppendDiagnostic("BRINIT\n");
	if (!resource.Init())
	{
		AppendDiagnostic("ERINIT -1\n");
		return FALSE; 
	}
	AppendDiagnostic("ERINIT 0\n");

	// Create manager
	AppendDiagnostic("BCMAN\n");
	if (!CreateManager())
	{
		AppendDiagnostic("ECMAN -1\n");
		return FALSE;
	}
	AppendDiagnostic("ECMAN 0\n");

	// Register plugins
	// set current dir to plugin path for messiahC4D.cfg file load
	_chdir( pluginDir );
	AppendDiagnostic("BRCFG\n");
	if( !ReadConfig() )
	{
		AppendDiagnostic("ERCFG -1\n");
		return FALSE;
	}
	AppendDiagnostic("ERCFG 0\n");

	AppendDiagnostic("BRCON\n");
	if (!RegisterMessiahConnection())
	{
		AppendDiagnostic("ERCON -1\n");
		return FALSE;
	}
	AppendDiagnostic("ERCON 0\n");

	AppendDiagnostic("BRTAG\n");
	if (!RegisterMessiahTag())
	{
		AppendDiagnostic("ERTAG -1\n");
		return FALSE;
	}
	AppendDiagnostic("ERTAG 0\n");

	// load messiahHOST.dll
	AppendDiagnostic("BLHST\n");
	if( !hostLoadmessiahHOST() )
	{
		_chdir( cwd );
		AppendDiagnostic("ELHST -1\n");
		return FALSE;
	}
	AppendDiagnostic("ELHST 0\n");
	_chdir( cwd );

	// start messiah
	AppendDiagnostic("BLMSA\n");
	if( !messiahStart(FX_HOST_APP_NAME, FX_HOST_MODULE_NAME, hostGlobal) ) 
	{
		AppendDiagnostic("ELMSA -1\n");
		return FALSE;
	}
	AppendDiagnostic("ELMSA 0\n");

	return TRUE;
}

void C4D_PlEnd(void)
{
	FreeMessiahConnection();
	FreeMessiahTag();
	FreeManager();

	// Notify messiah that we're out of here
	hostUnLoadmessiahHOST();

//	s->UnLock();
	FreeSemaphore(s);
	_chdir( pluginDir );
	WriteConfig();
	_chdir( cwd );

	EndDiagnostic();
}
