#ifndef MESSIAH_MANAGER
#define MESSIAH_MANAGER


#include "c4d_baselist.h"
#include "c4d_baseobject.h"
#include "c4d_basedocument.h"
#include "c4d_general.h"

#include "messiahUtilities.h"
#include "messiahDLL.h"

#include <vector>
#include <algorithm>
#ifdef _MSC_VER
  #pragma warning(disable : 4786)
#endif


// Singleton manager for the messiah connection
class messiahManager
{
public:
  messiahManager() : m_justLoaded(FALSE), m_fps(30) {}

  // Tells messiah to save the current state to the hyperfile.
  // The path is used when saving to file; it should point to the doc directory.
  Bool saveScene(HyperFile& hf, const Filename& path);

  // Save the node number to the hyperfile
  // ron
  Bool saveTags(HyperFile &hf, PluginObject *op);
  Bool loadTags(HyperFile &hf);
  
  // Tells messiah to load the current state from a hyperfile.
  // (Automatically disables loadGeometry and objectAdd.)
  Bool loadScene(HyperFile& hf);

  // Opens the messiah interface.
  void interfaceRequested();

  // Tells messiah how the current set of objects look like.
  // (The function scans the hierarchy for tagged objects, 
  // adding those missing to messiah and removing those
  // that are no longer there.)
  void notifyCurrentHierarchy(BaseObject* parent);

  // Tells messiah to process the object with messiah:displace and :motion.
  void processObject(BaseObject* obj, const BaseTime& t);

  // Gets the current geometry for a messiah_node->element. (Used by loadGeometry.)
  BaseObject* getGeometryFromElement(void* element);

  LONG m_fps;

private:
  Bool m_justLoaded;

  struct connectionData
  {
    connectionData() : m_messiahNode(NULL), m_c4dNode(NULL), m_processed(FALSE) {}
    ~connectionData() { FreeObject(m_c4dNode); }

    FX_NodeLog* m_messiahNode;
    BaseObject* m_c4dNode;
    Bool m_processed;

    void replaceC4DNode(BaseObject* newNode)
    {
      FreeObject(m_c4dNode);
      m_c4dNode = newNode->GetClone();
    }

    // For use in the algorithm
    static bool hasBeenProcesed(connectionData* data) {return data->m_processed == TRUE; }
    static bool hasNotBeenProcesed(connectionData* data) {return data->m_processed == FALSE; }

  private:
    // It would be bad to copy this structure
    connectionData(const connectionData& forbidden) {}
    void operator = (const connectionData& forbidden) {}
  };

public:
  typedef std::vector<connectionData*> connectionPtrs;
  connectionPtrs m_connections;

  typedef std::vector<LONG>	loadIDs;
  loadIDs		m_loadIDs;

  connectionData* getConnectionFromObject(BaseObject* obj)
  {
    if (!obj) return NULL;

    for (int i = 0; i < m_connections.size(); ++i)
    {
      if (m_connections[i]->m_c4dNode && 
          m_connections[i]->m_c4dNode->GetName() == obj->GetName())
      {
        return m_connections[i];
      }
    }
    return NULL;
  }

  void resetFlags()
  {
    for (int i = 0; i < m_connections.size(); ++i)
    {
      m_connections[i]->m_processed = FALSE;
    }
  }


};

extern messiahManager* mm;

#endif

